Package: PICB
Title: piRNA Cluster Builder
Version: 1.4.0
Authors@R: c(person("Pavol", "Genzor",  role = c("aut")),
    person("Aleksandr", "Friman", email="haase-lab-bioinfo@nih.gov",  role = c("aut")),
    person("Daniel", "Stoyko", role = c("aut")),
    person("Parthena", "Konstantinidou", email="haase-lab-bioinfo@nih.gov", role = c("aut")),
    person("Franziska", "Ahrend", email="haase-lab-bioinfo@nih.gov",  role = c("aut", "cre"), comment=c(ORCID = "0009-0004-7464-3444")),
    person("Zuzana", "Loubalova", email="haase-lab-bioinfo@nih.gov",  role = c("aut")), 
    person("Yuejun", "Wang", email="haase-lab-bioinfo@nih.gov",  role = c("aut")),
    person("Hernan", "Lorenzi", email="haase-lab-bioinfo@nih.gov",  role = c("aut")),
    person("Astrid D", "Haase", email="haase-lab-bioinfo@nih.gov",  role = c("aut")))
Description: piRNAs (short for PIWI-interacting RNAs) and their PIWI protein partners play a key role in fertility 
    and maintaining genome integrity by restricting mobile genetic elements (transposons) in germ cells. 
    piRNAs originate from genomic regions known as piRNA clusters. The piRNA Cluster Builder (PICB) is 
    a versatile toolkit designed to identify genomic regions with a high density of piRNAs. It constructs 
    piRNA clusters through a stepwise integration of unique and multimapping piRNAs and offers wide-ranging 
    parameter settings, supported by an optimization function that allows users to test different parameter 
    combinations to tailor the analysis to their specific piRNA system. The output includes extensive metadata 
    columns, enabling researchers to rank clusters and extract cluster characteristics. 
License: CC0
Encoding: UTF-8
Imports: utils, Seqinfo, GenomicRanges, GenomicAlignments, Biostrings,
        Rsamtools, data.table, IRanges, seqinr, stats, openxlsx, dplyr,
        S4Vectors, methods
Suggests: GenomeInfoDb, knitr, rtracklayer, testthat, BiocStyle,
        prettydoc, BSgenome, BSgenome.Dmelanogaster.UCSC.dm6,
        BiocManager, rmarkdown, ggplot2
VignetteBuilder: knitr
BugReports: https://github.com/HaaseLab/PICB/issues
URL: https://github.com/HaaseLab/PICB
biocViews: Genetics, GenomeAnnotation, Sequencing,
        FunctionalPrediction, Coverage, Transcriptomics
RoxygenNote: 7.3.2
git_url: https://git.bioconductor.org/packages/PICB
git_branch: RELEASE_3_23
git_last_commit: e2dfa54
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 23:53:36 UTC; biocbuild
Author: Pavol Genzor [aut],
  Aleksandr Friman [aut],
  Daniel Stoyko [aut],
  Parthena Konstantinidou [aut],
  Franziska Ahrend [aut, cre] (ORCID:
    <https://orcid.org/0009-0004-7464-3444>),
  Zuzana Loubalova [aut],
  Yuejun Wang [aut],
  Hernan Lorenzi [aut],
  Astrid D Haase [aut]
Maintainer: Franziska Ahrend <haase-lab-bioinfo@nih.gov>
