# Generated by roxygen2: do not edit by hand

export(PICBannotate)
export(PICBbuild)
export(PICBexporttoexcel)
export(PICBgetchromosomes)
export(PICBimportfromexcel)
export(PICBload)
export(PICBloadfasta)
export(PICBoptimize)
export(PICBstrandanalysis)
